import {
  select,
  csv,
  scaleLinear,
  scaleTime,
  scaleOrdinal,
  extent,
  axisLeft,
  axisBottom,
  line,
  curveBasis,
  timeParse,
  nest,
  max,
  schemeCategory10
} from 'd3';

import { colorLegend } from './colorLegend';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

const render = data => {
  const title = 'Body Weight';
  
  const xValue = d => d.date;
  const xAxisLabel = 'Date';
  
  const yValue = d => d.weight;
  const circleRadius = 3;
  const yAxisLabel = 'Weight (kg)';
  
  const colorValue = d => d.person;
  
  const margin = { top: 60, right: 40, bottom: 88, left: 105 };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  
  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();
  
  const yScale = scaleLinear()
    //.domain([0, max(data, yValue)])
    .domain([45, 110])
    .range([innerHeight, 0])
    .nice();
  
  const colorScale = scaleOrdinal(schemeCategory10);
  
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left},${margin.top})`);
  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15);
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);
  
  const yAxisG = g.append('g').call(yAxis);
  yAxisG.selectAll('.domain').remove();
  
  yAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', -60)
      .attr('x', -innerHeight / 2)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
      .text(yAxisLabel);
  
  const xAxisG = g.append('g').call(xAxis)
    .attr('transform', `translate(0,${innerHeight})`);
  
  xAxisG.select('.domain').remove();
  
  xAxisG.append('text')
      .attr('class', 'axis-label')
      .attr('y', 80)
      .attr('x', innerWidth / 2)
      .attr('fill', 'black')
      .text(xAxisLabel);
  
  const lineGenerator = line()
    .x(d => xScale(xValue(d)))
    .y(d => yScale(yValue(d)))
    .curve(curveBasis);
  
  const nestedData = nest()
    .key(colorValue)
    .entries(data);
  colorScale.domain(nestedData.map(d => d.key))
  
  g.selectAll('.line-path').data(nestedData)
    .enter().append('path')
      .attr('class', 'line-path')
      .attr('d', d => lineGenerator(d.values))
      .attr('stroke', d => colorScale(d.key));
  
  g.append('text')
      .attr('class', 'title')
      .attr('y', -10)
      .text(title);

  svg.append('g')
    .attr('transform', `translate(130,370)`)
    .call(colorLegend, {
      colorScale,
      circleRadius: 8,
      spacing: 25,
      textOffset: 14
    });

  
  
  let circle1 = g.append('circle')
    .attr('r', '3');
  let circle2 = g.append('circle')
    .attr('r', '3');
  let paths = document.getElementsByClassName('line-path');
  let path1 = paths[0];
  let path2 = paths[1];
  let totalLength1 = path1.getTotalLength();
  let totalLength2 = path2.getTotalLength();
  
  let range = document.querySelector('input');
  let maxLimit = 5000
range.setAttribute('max', maxLimit);
range.setAttribute('step', 1);
  
  range.addEventListener('input', function(e){
    var pos1 = path1.getPointAtLength(e.srcElement.value * totalLength1 / maxLimit);
    circle1.attr('cx', pos1.x);
    circle1.attr('cy', pos1.y);
    var pos2 = path2.getPointAtLength(e.srcElement.value * totalLength2 / maxLimit);
    circle2.attr('cx', pos2.x);
    circle2.attr('cy', pos2.y);

  });
  
};

csv('weight.csv')
  .then(data => {
  const parseDate = timeParse("%d/%m/%Y");
    data.forEach(d => {
      d.weight = +d.weight;
      d.date = parseDate(d.date);
    });
    render(data);
  });
